#!/usr/local/bin/perl -w
#
# pure perl httpd. (c) Paul Tchistopolskii 1998, 99
# http://www.pault.com  e-mail: paul@qub.com
#
# Public domain. I'l appreciate any feedback.
# Especialy ideas how to remove a couple of lines from this file.
#
# Apr.29.1998: v 0.2
#	'Support' for 'POST'. It was useful for debugging uploads.
#
# Nov.27.1998: v 0.1
#       Supports only .htm(l) and .cgi in one directory
#       Not all ENV's are set. No 'index.html'. Only 'GET' yet.
#
#

use strict; use Socket;

my $port = 10080;
my $PERL = 'c:\\perl\\bin\\perl.exe';
my $content_len;
my $buf;
my $auth;

$|=1;

sub logmsg { print STDERR scalar localtime, ": $$: @_\n"; }

my %codes = (
	'200', 'OK',
	'201', 'Created',
	'202', 'Accepted',
	'204', 'No Content',
	'301', 'Moved Permanently',
	'302', 'Moved Temporarily',
	'304', 'Not Modified',
	'400', 'Bad Request',
	'401', "Unauthorized\nWWW-Authenticate: Basic realm=\"phttpd\"",
	'403', 'Forbidden',
	'404', 'Not Found',
	'500', 'Internal Server Error',
	'501', 'Not Implemented',
	'502', 'Bad Gateway',
	'503', 'Service Unavailable',
);

sub logerr ($$) { 
	my ($code, $detail) = @_;
	my $msg = "$code " . $codes{$code};
	logmsg "$detail : $msg";
	print STDERR "HTTP/1.0 $msg\nContent-type: text/html\n\n";
	print STDERR "<I>PurePerlHttpd</I> : $detail : $msg\n";
	print  "HTTP/1.0 $msg\nContent-type: text/html\n\n";
	print  "<I>PurePerlHttpd</I> : $detail : $msg\n";
}

sub cat($){
	my $file = shift;
	open IN, "<$file" || return 0;
	my $content = join '', <IN>;	
	close IN;

	print "HTTP/1.0 200 OK\n";
	print "Content-type: text/html\n\n";
	print $content;
	return 1;
}


my $tcp = getprotobyname('tcp');
socket(Server, PF_INET, SOCK_STREAM, $tcp)      || die "socket: $!";
setsockopt(Server, SOL_SOCKET, SO_REUSEADDR,
                  pack("l", 1))   || die "setsockopt: $!";
bind(Server, sockaddr_in($port, INADDR_ANY))    || die "bind: $!";
listen(Server,SOMAXCONN)                        || die "listen: $!";

logmsg "server started on port $port";

my $addr; my @inetaddr;

for ( ; $addr = accept(Client,Server); close Client) {
	my($undef, $undef, $inetaddr) = unpack('S n a4 x8', $addr);
	@inetaddr = unpack('C4', $inetaddr);

	logmsg "incoming connection from: " , join(".", @inetaddr);

	*STDIN = *Client;
	*STDOUT = *Client;

	$_ = <STDIN>;
	print STDERR $_;
	my ($method, $url, $proto, $garbage) = split; #GET /t.cgi HTTP/1.0

	if ($garbage ne '') { 
		logerr 400, $_;
		exit;
	}

	logmsg "Req: mthd=$method, url=$url, prot=$proto";
	$url =~ s/%([\dA-Fa-f]{2})/chr(hex($1))/eg; # unescape.
	logmsg "Unescaped url: $url";

	$content_len = 0;
	$auth = "";
	if ( $method ne 'GET') {
		if ( $method ne 'POST' ) {		
			logerr 501, $method;
			exit;
		}
	}
	while ( <STDIN> ) {
		print STDERR $_;
		if ($_ =~ /^Authorization: Basic (\S+)\r\n$/) { $auth = $1;}
		if ($_ =~ /^Content-Length: (\d+)\r\n$/) { $content_len = $1;}
		if ($method eq "POST" && $_ eq "\r\n") { read(STDIN,$buf,$content_len); print STDERR $buf; last;}
		last if ($_ eq "\r\n");
	}
	print STDERR "\n";
	if ( $url !~ m|.*/([^/]*)$| ) {
		logerr 400, $url;
	} else {
		my $file = $1;
		$ENV{'REQUEST_URI'}       = $url;
		$ENV{'SERVER_PROTOCOL'}   = $proto;
		$ENV{'REQUEST_METHOD'}    = $method;
		$ENV{'REMOTE_ADDR'}       = $inetaddr;
		if ( not -e $file ) {
			logerr 404, $file;
		} else {
			if ( $file =~ m/\.cgi$/ ) {
				logmsg "Executing '$file'";
				my $res = `$PERL $file`; # Win!
				logmsg "Sending results...";
				print "HTTP/1.0 200 OK\n";
				print $res;
			} elsif ( $file =~ m/\.html?$/ ) {
				if ($auth ne "") {
					logmsg "Dumping '$file'";
					cat $file || logerr 500,$file;
				} else {
					logerr 401, "auth error";
				}
			} else {
				logerr 501, $file;
			}
		}
	}

	close STDIN;
	close STDOUT;
} #for loop


